/*
libSensPlugin.so,
a library to count libsensors values in VampirTrace
Copyright (C) 2010 TU Dresden, ZIH

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

 */
#include <vampirtrace/vt_plugin_cntr.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sensors/sensors.h>

#define MAX_SENSOR_COUNTER 40

typedef struct lm_sensors_counter_t {
  const sensors_subfeature * subFeature;
  const sensors_chip_name * chip;
} sensor_counter;

static sensor_counter sensor_counters[MAX_SENSOR_COUNTER];
static int used_sensor_counters = 0;

int32_t init()
{
  char * sensors_file_path;
  FILE * sensors_file=NULL;
  /* check whether sensors config file is given */
  sensors_file_path = getenv("VT_PLUGIN_CNTR_LM_SENSORS_FILE");
   if (sensors_file_path == NULL) {
     sensors_file=fopen(sensors_file_path,"r");
   }
  sensors_init(sensors_file);
  return 0;
}

vt_plugin_cntr_metric_info * get_event_info(char * event_name)
{
    vt_plugin_cntr_metric_info * return_values;
    /* Wildcards are not supported yet, so we need two entries */
    return_values = malloc(2 * sizeof(vt_plugin_cntr_metric_info));

    /* if the description is null it should be considered the end */
    return_values[0].name = strdup(event_name);
    return_values[0].unit = NULL;
    return_values[0].cntr_property = VT_PLUGIN_CNTR_ABS
            | VT_PLUGIN_CNTR_DOUBLE | VT_PLUGIN_CNTR_POINT;
    /* Last element empty */
    return_values[1].name = NULL;
    return return_values;
}

void fini()
{
  sensors_cleanup();
}

int32_t add_counter(char * event_name)
{
  int chipNr=0;
  int featureNr=0;
  int subFeatureNr=0;
  const sensors_subfeature * tmp_subFeature;
  const sensors_feature * tmp_feature;
  const sensors_chip_name * tmp_chip;
  char buffer[256];
  if (used_sensor_counters==MAX_SENSOR_COUNTER){
    fprintf(stderr,"Ignoring event '%s', only %i events supported\n"
        "Adopt MAX_SENSOR_COUNTER and recompile the sensors plugin\n"
        "to avoid this limitation.\n"
        ,event_name, MAX_SENSOR_COUNTER);
  }
  memset(buffer,0,256);
  tmp_chip = sensors_get_detected_chips(NULL, &chipNr);
  // while theres a new chip
  for(;tmp_chip!=NULL;tmp_chip = sensors_get_detected_chips(NULL, &chipNr)) {
    featureNr=0;
    tmp_feature = sensors_get_features(tmp_chip, &featureNr);
      // while theres a new feature
      for (;tmp_feature!=NULL;tmp_feature = sensors_get_features(tmp_chip, &featureNr)){
        subFeatureNr=0;
        tmp_subFeature=sensors_get_all_subfeatures(tmp_chip,tmp_feature,&subFeatureNr);
          // while theres a new subfeature
          for (;tmp_subFeature!=NULL;tmp_subFeature=sensors_get_all_subfeatures(tmp_chip,tmp_feature,&subFeatureNr))
          {
            sprintf(buffer, "%s/%s/%s", tmp_chip->prefix,sensors_get_label(tmp_chip,tmp_feature),tmp_subFeature->name);
            //fprintf(stderr,"%s\n",buffer);
            if (strcmp(buffer, event_name)==0){
              sensor_counters[used_sensor_counters].chip=tmp_chip;
              sensor_counters[used_sensor_counters].subFeature=tmp_subFeature;
              used_sensor_counters++;
              return used_sensor_counters-1;
            }
          }
      }
    }
  return -1;
}

uint64_t get_current_value(int32_t ID){
  int retVal=0;
  union{
    double dbl;
    uint64_t uint64;
  } value;
  retVal=sensors_get_value(sensor_counters[ID].chip,
                          sensor_counters[ID].subFeature->number,
      &value.dbl);

  return value.uint64;
}


vt_plugin_cntr_info get_info()
{
    vt_plugin_cntr_info info;
    memset(&info,0,sizeof(vt_plugin_cntr_info));
    info.init                     = init;
    info.add_counter              = add_counter;

    info.vt_plugin_cntr_version   = VT_PLUGIN_CNTR_VERSION;
    info.run_per                  = VT_PLUGIN_CNTR_PER_HOST;
    info.synch                    = VT_PLUGIN_CNTR_SYNCH;
    info.get_event_info           = get_event_info;
    info.get_current_value        = get_current_value;
    info.finalize                 = fini;
    return info;
}
